// ----------------------------------------------------------------------------
// markItUp!
// ----------------------------------------------------------------------------
// Copyright (C) 2008 Jay Salvat
// http://markitup.jaysalvat.com/
// ----------------------------------------------------------------------------
// Html tags
// http://en.wikipedia.org/wiki/html
// ----------------------------------------------------------------------------
// Basic set. Feel free to add more tags
// ----------------------------------------------------------------------------
wikiSettings = {
    nameSpace: 'wiki',
    onShiftEnter:      {keepDefault:false, replaceWith:'<br />\n'},
    onCtrlEnter:      {keepDefault:false, openWith:'\n<p>', closeWith:'</p>'},
    onTab:            {keepDefault:false, replaceWith:'    '},
    previewTemplatePath:    '/utility/markitup_preview',
    previewParserPath: '/utility/markitup?format=wiki',
    previewPosition: 'after',
    markupSet:  [
        {name:'Bold', key:'B', openWith:'(!(<strong>|!|<b>)!)', closeWith:'(!(</strong>|!|</b>)!)' },
        {name:'Italic', key:'I', openWith:'(!(<em>|!|<i>)!)', closeWith:'(!(</em>|!|</i>)!)'  },
        {name:'Stroke through', key:'S', openWith:'<del>', closeWith:'</del>' },
        {separator:'---------------' },
        {name:'Picture', key:'P', replaceWith:'<img src="[![Source:!:http://]!]" alt="[![Alternative text]!]" />' },
        {name:'Link', key:'L', openWith:'<a href="[![Link:!:http://]!]"(!( title="[![Title]!]")!)>', closeWith:'</a>', placeHolder:'Your text to link...' },
        {separator:'---------------' },
        {name:'Clean', className:'clean', replaceWith:function(markitup) { return markitup.selection.replace(/<(.*?)>/g, "") } },        
        {name:'Preview', className:'preview',  call:'preview'}
    ]
};

markdownSettings = {
    nameSpace: 'markdown',
    onShiftEnter:      {keepDefault:false, replaceWith:'<br />\n'},
    onCtrlEnter:      {keepDefault:false, openWith:'\n<p>', closeWith:'</p>'},
    onTab:            {keepDefault:false, replaceWith:'    '},
    previewTemplatePath:    '/utility/markitup_preview',
    previewParserPath: '/utility/markitup?format=markdown',
    previewPosition: 'after',
    markupSet:  [
        {name:'Bold', key:'B', openWith:'(!(<strong>|!|<b>)!)', closeWith:'(!(</strong>|!|</b>)!)' },
        {name:'Italic', key:'I', openWith:'(!(<em>|!|<i>)!)', closeWith:'(!(</em>|!|</i>)!)'  },
        {name:'Stroke through', key:'S', openWith:'<del>', closeWith:'</del>' },
        {separator:'---------------' },
        {name:'Picture', key:'P', replaceWith:'<img src="[![Source:!:http://]!]" alt="[![Alternative text]!]" />' },
        {name:'Link', key:'L', openWith:'<a href="[![Link:!:http://]!]"(!( title="[![Title]!]")!)>', closeWith:'</a>', placeHolder:'Your text to link...' },
        {separator:'---------------' },
        {name:'Clean', className:'clean', replaceWith:function(markitup) { return markitup.selection.replace(/<(.*?)>/g, "") } },        
        {name:'Preview', className:'preview',  call:'preview'}
    ]
};
